/*
 * Copyright (C) 2009 Red Hat, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 * Author: David Zeuthen <davidz@redhat.com>
 */

#if !defined (_POLKIT_GTK_MATE_INSIDE_POLKIT_GTK_MATE_H) && !defined (POLKIT_GTK_MATE_COMPILATION)
#error "Only <polkitgtkmate/polkitgtkmate.h> can be included directly, this file may disappear or change contents."
#endif

#ifndef __POLKIT_GTK_MATE_TYPES_H
#define __POLKIT_GTK_MATE_TYPES_H

#include <gtk/gtk.h>
#include <polkit/polkit.h>

struct _PolkitLockButton;
typedef struct _PolkitLockButton PolkitLockButton;

#endif /* __POLKIT_GTK_MATE_TYPES_H */
